package com.qmth.themis.business.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.ExamCacheBean;
import com.qmth.themis.business.cache.bean.ExamStudentAnswerCacheBean;
import com.qmth.themis.business.cache.bean.ObjectiveAnswerCacheBean;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dao.TOeExamRecordMapper;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.enums.ExamRecordStatusEnum;
import com.qmth.themis.business.enums.ObjectiveScorePolicyEnum;
import com.qmth.themis.business.service.TEExamPaperService;
import com.qmth.themis.business.service.TEExamService;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.util.SimpleBeanUtil;

/**
 * @Description: 考试记录 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TOeExamRecordServiceImpl extends ServiceImpl<TOeExamRecordMapper, TOeExamRecord> implements TOeExamRecordService {

    @Resource
    TOeExamRecordMapper tOeExamRecordMapper;

    @Resource
    RedisUtil redisUtil;
    
	@Resource
	TEExamPaperService examPaperService;

	@Resource
	TEExamService examService;
    
    /**
     * 获取考试未完列表
     *
     * @param studentId
     * @param examId
     * @param orgId
     * @return
     */
    @Override
    public Map getUnFinishExam(Long studentId, Long examId, Long orgId) {
        Map unFinishExam = tOeExamRecordMapper.getUnFinishExam(studentId, examId, orgId);
        if (Objects.nonNull(unFinishExam)) {
            Map finalMap = new HashMap();
            Map<String, Object> waitingMap = new HashMap();
            waitingMap.put("id", unFinishExam.get("id"));
            waitingMap.put("examActivityId", unFinishExam.get("examActivityId"));
            waitingMap.put("name", unFinishExam.get("name"));
            waitingMap.put("mode", unFinishExam.get("mode"));
            waitingMap.put("preNotice", unFinishExam.get("preNotice"));
            waitingMap.put("preNoticeStaySeconds", unFinishExam.get("preNoticeStaySeconds"));
            waitingMap.put("postNotice", unFinishExam.get("postNotice"));
            finalMap.put("waiting", waitingMap);

            Map<String, Object> activityMap = new HashMap();
            activityMap.put("recordId", unFinishExam.get("recordId"));
            activityMap.put("minDurationSeconds", unFinishExam.get("minDurationSeconds"));
            activityMap.put("clientVideoPush", unFinishExam.get("clientVideoPush"));
            activityMap.put("cameraPhotoUpload", unFinishExam.get("cameraPhotoUpload"));
            activityMap.put("courseCode", unFinishExam.get("courseCode"));
            activityMap.put("wxappPhotoUpload", unFinishExam.get("wxappPhotoUpload"));
            activityMap.put("inProcessLivenessRetryCount", unFinishExam.get("inProcessLivenessRetryCount"));
            activityMap.put("examStudentId", unFinishExam.get("examStudentId"));
            activityMap.put("inProcessLivenessVerify", unFinishExam.get("inProcessLivenessVerify"));
            activityMap.put("inProcessFaceStrangerIgnore", unFinishExam.get("inProcessFaceStrangerIgnore"));
            activityMap.put("wxappVideoRecord", unFinishExam.get("wxappVideoRecord"));
            activityMap.put("inProcessFaceVerify", unFinishExam.get("inProcessFaceVerify"));
            activityMap.put("courseName", unFinishExam.get("courseName"));
            activityMap.put("wxappVideoPush", unFinishExam.get("wxappVideoPush"));
            activityMap.put("inProcessLivenessFixedRange", unFinishExam.get("inProcessLivenessFixedRange"));
            activityMap.put("prepareSeconds", unFinishExam.get("prepareSeconds"));
            activityMap.put("fouceFinish", unFinishExam.get("fouceFinish"));
            activityMap.put("entryAuthenticationPolicy", unFinishExam.get("entryAuthenticationPolicy"));
            activityMap.put("maxDurationSeconds", unFinishExam.get("maxDurationSeconds"));
            activityMap.put("examCount", unFinishExam.get("examCount"));
            activityMap.put("inProcessLivenessJudgePolicy", unFinishExam.get("inProcessLivenessJudgePolicy"));
            activityMap.put("id", unFinishExam.get("examActivityId"));

            finalMap.put("activity", activityMap);
            return finalMap;
        } else {
            return unFinishExam;
        }
    }

    @Transactional
	@Override
	public Long saveByPrepare(Long examId, Long examActivityId, Long examStudentId, Long paperId,
			Integer serialNumber) {
		TOeExamRecord er = new TOeExamRecord();
		er.setId(Constants.idGen.next());
		er.setExamId(examId);
		er.setExamActivityId(examActivityId);
		er.setExamStudentId(examStudentId);
		er.setPaperId(paperId);
		er.setSerialNumber(serialNumber);
		er.setFirstPrepareTime(new Date());
		er.setStatus(ExamRecordStatusEnum.FIRST_PREPARE);
		redisUtil.set(RedisKeyHelper.examRecordCacheKey(er.getId()), SimpleBeanUtil.objectToMap(er));
		return er.getId();
	}

	/**
	 *计算客观分
	 */
	@Override
	public void calculateObjectiveScore(Map<String, Object> param) {
		Long recordId = (Long) param.get("recordId");
		Integer mainNumber = (Integer) param.get("mainNumber");
		Integer subNumber = (Integer) param.get("subNumber");
		Integer subIndex = (Integer) param.get("subIndex");
		Long paperId = ExamRecordCacheUtil.getPaperId(recordId);
		Map<String, ObjectiveAnswerCacheBean> map = examPaperService.getObjectiveAnswerCacheBean(paperId);
		if (map == null || map.size() == 0) {
			log.error("no ObjectiveAnswerCacheBean for calculateObjectiveScore");
			return;
		}
		String key = RedisKeyHelper.examAnswerHashKey(mainNumber, subNumber, subIndex);
		ObjectiveAnswerCacheBean cb = map.get(key);
		if(cb==null) {
			return;
		}
		ExamStudentAnswerCacheBean answer = (ExamStudentAnswerCacheBean) redisUtil.get(
				RedisKeyHelper.examAnswerKey(recordId),
				RedisKeyHelper.examAnswerHashKey(mainNumber, subNumber, subIndex));
		if (answer == null) {
			log.error("no ExamStudentAnswerCacheBean for calculateObjectiveScore");
			return;
		}

		if (cb.getStructType().intValue() == 1) {
			if (checkSingleChoice(answer.getContent(), cb.getChoiceAnswer())) {
				answer.setScore(cb.getScore());
			} else {
				answer.setScore(0.0);
			}
		}
		if (cb.getStructType().intValue() == 2) {
			ExamCacheBean ec = examService.getExamCacheBean(ExamRecordCacheUtil.getExamId(recordId));
			int r = checkMultipleChoice(answer.getContent(), cb.getChoiceAnswer());
			if (ec.getObjectiveScorePolicy().equals(ObjectiveScorePolicyEnum.EQUAL)) {// 全对给分
				if (r == 1) {
					answer.setScore(cb.getScore());
				} else {
					answer.setScore(0.0);
				}
			}
			if (ec.getObjectiveScorePolicy().equals(ObjectiveScorePolicyEnum.PARTIAL)) {// 漏选半分
				if (r == 1) {
					answer.setScore(cb.getScore());
				} else if (r == 0) {
					BigDecimal b = new BigDecimal(cb.getScore()).divide(new BigDecimal("2"), 1, BigDecimal.ROUND_UP);
					answer.setScore(b.doubleValue());
				} else {
					answer.setScore(0.0);
				}
			}
		}
		if (cb.getStructType().intValue() == 3) {
			if (Boolean.parseBoolean(answer.getContent())==cb.getBoolAnswer().booleanValue()) {
				answer.setScore(cb.getScore());
			} else {
				answer.setScore(0.0);
			}
		}

		answer.setObjective(true);
		// 更新分数
		redisUtil.set(RedisKeyHelper.examAnswerKey(recordId),
				RedisKeyHelper.examAnswerHashKey(mainNumber, subNumber, subIndex), answer);

		// 计算客观分总分
		calculateTotalObjectiveScore(recordId, map);
	}

	@SuppressWarnings("unchecked")
	private void calculateTotalObjectiveScore(Long recordId, Map<String, ObjectiveAnswerCacheBean> map) {
		String lockKey = SystemConstant.REDIS_LOCK_TOTAL_OBJECTIVE_SCORE_PREFIX + recordId;
		try {
			Boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_CACHE_TIME_OUT);
			if (lock) {
				Map<String,ExamStudentAnswerCacheBean> as=redisUtil.getHashEntries(RedisKeyHelper.examAnswerKey(recordId));
				if(as!=null&&as.size()>0) {
					Double total=0.0;
					for(ExamStudentAnswerCacheBean sa:as.values()) {
						if(sa.getObjective()!=null&&sa.getObjective()) {//是客观题
							if(sa.getScore()!=null) {//有分值
								total=total+sa.getScore();
							}else {
								total=null;
								break;
							}
						}
					}
					if(total!=null) {
						ExamRecordCacheUtil.setObjectiveScore(recordId, total);
					}
				}
			}
		} finally {
			redisUtil.releaseLock(lockKey);
		}
	}

	private boolean checkSingleChoice(String answer, JSONArray ar) {
		int a = Integer.parseInt(answer);
		for (int i = 0; i < ar.size(); i++) {
			if (a == ar.getIntValue(i)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @param answer
	 * @param ar
	 * @return -1 有错选项，0-无措选项 未答全 1-全对
	 */
	private int checkMultipleChoice(String answer, JSONArray ar) {
		int yes = 0;
		int no = 0;
		String[] ss = answer.split(",");
		for (String an : ss) {
			if (checkSingleChoice(an, ar)) {
				yes++;
				continue;
			} else {
				no++;
			}
		}
		if (yes == ar.size()) {
			return 1;
		}
		if (yes != 0 && no == 0) {
			return 0;
		}
		return -1;
	}
}
